/*===============================================================================*/
/* Determining TAR order.                                                        */
/* Test used - LM1                                                               */
/* ESTIMATE A PURE THRESHOLD MODEL                                               */
/* minimal regime size requirement 10%                                           */
/* grid search cut from ends  10%                                                */
/* control all (C1,C2,C3) estimates with 0.1 and 0.9 quantiles                   */
/*===============================================================================*/
cls;
declare level, _trim;

library pgraph;
chdir c:\tarspec\inpaper\unempl;
output file=c:\tarspec\citibase-final-longdif4.out reset;

algus=time;

format /m1 /rz 8,4; 
 
load qt=citibase;
y = trimr(qt,1,0) - trimr(lag1(qt),1,0);

/* common parameters */
repl=1;                           /* Nr. of replications */                         
level_set={0.05};
_trim=0.10;
_qnum=0;

npp=12;
lagi=3;
n=rows(y);
ylag=zeros(n-npp,npp);

i=1;
do while i<=npp;
   ylag[.,i]=y[npp-i+1:n-i,1];                  /* All lags */
   i=i+1;
endo;
format /rds 8,3;

_y_=y[npp+1:n,1];
yks=ones(rows(_y_),1);
_X_=yks~ylag;   /* Linear model is AR(12) */

trans=(cumsumc(_X_[.,2:cols(_X_)]'))'; 

"Alfa=" level_set;

output on;
"********************************************************";
"d="  lagi+1;
_tvar_=trans[.,lagi];                       /* transition variable */


    
mrs=round(_trim*rows(_y_));  /* min # of obs in a regime */
T=rows(_y_);


for l (1, rows(level_set), 1);
level=level_set[l]; 

gam1hat=zeros(repl,1);
break1=zeros(repl,1);
beta1=zeros(repl,cols(_X_));
beta2=zeros(repl,cols(_X_));
r1hatit=zeros(1,1);
r2hatit=zeros(1,1);


   blin=(invpd(_X_'_X_)*_X_'_y_);
   u2lin=(_y_-_X_*blin)'(_y_-_X_*blin);
   "SSE linear=" u2lin;


   c_m=zeros(1,6);
   breaks=zeros(1,5);
   em=0;       
   _auxx=(_tvar_.*_X_[.,2:cols(_X_)]);  /* aux.regressors x_t*s_t */
   area=qsort(_tvar_); 

     {gam1hat,beta1,beta2,u}=tar(_y_,_X_,_tvar_,area);         
      SSE=u'u;
      "SSE=" SSE;

      
/*1.level*/ 
       {LM1_F,LM1_p}=LIN_TEST(_y_,_X_,_auxx);  
            
       if LM1_p >=level;                    /* if linearity is NOT rejected */
          c_m[1,em+1]=c_m[1,em+1]+1;
         screen on; print"NO 1.BREAK,pval=" LM1_p;     
        else;                              /* If linearity IS rejected */
         
          format /m1 8,6; 
          "p-value for linearity test=" LM1_p;
          
          /* --- estimate TAR(1) --- */
          u=zeros(rows(_y_),1);
          {gam1hat,beta1,beta2,u}=tar(_y_,_X_,_tvar_,area);          
          
          
          _plctrl={0,0,0};
          _pltype={6,2,5};
          _plegctl=1;
          _plegstr="data\000residuals\000fit TAR1";
          xy(seqa(1,1, rows(u)),_y_~u~(_y_-u));
          _plegctl=0;
          _plegstr="";
          
          break1[1,1]=gam1hat[1];
          "C1=" gam1hat[1];          
          em=em+1;                       /* If valid break, take into account */
          breaks[1,1]= gam1hat[1];
           
          x1hat=_X_.*(_tvar_.<=gam1hat[1]);
          x2hat=_X_.*(_tvar_.>gam1hat[1]);
          reg=x1hat~x2hat;
                  
/*2.level*/
          {LM12_F,LM12_p}=TAR_TEST(_y_,reg,_auxx);
          
          if LM12_p >=level*tau;                /* if NO SECOND BREAK */
             c_m[1,em+1]=c_m[1,em+1]+1;     /* save em-count */
           "plm1for the second break=" LM12_p;
            
             goto replloop;               /* take next replication */
          else;
            "plm1for the second break=" LM12_p;
             em=em+1;
             format /m1 8,3;
            
             
             /* ----- Estimate LSTR2 ---- */  
            {r1,r2,r1it,r2it}=tar2(_y_,_X_,_tvar_);
                    
             r1hatit[1,1]=minc(r1it|r2it);
             r2hatit[1,1]=maxc(r1it|r2it);
              "New itrated thresholds" r1hatit~r2hatit;   
                      
             /* recuperating the betahats (based on the iterated ) */
             z1=_X_.*(_tvar_.<=r1hatit[1,1]);
             z2=_X_.*(_tvar_.>r1hatit[1,1] .and _tvar_.<=r2hatit[1,1]);
             z3=_X_.*(_tvar_.>r2hatit[1,1]);

             reg2=z1~z2~z3;               b1hat=(invpd(z1'z1)*z1'_y_)';
             b2hat=(invpd(z2'z2)*z2'_y_)';
             b3hat=(invpd(z3'z3)*z3'_y_)';
             bethat=(b1hat~b2hat~b3hat)';
             
             
             u2=(_y_-reg2*bethat)'(_y_-reg2*bethat);
             "SSE2=" u2;
             breaks[1,1:2]= r1hatit[1,1]~r2hatit[1,1];
             
             
          _plctrl={0,0,0};
          _pltype={6,2,5};
          _plegctl=1;
          _plegstr="data\000residuals\000fit TAR2";
          xy(seqa(1,1, rows(u)),_y_~(_y_-reg2*bethat)~(reg2*bethat));
          _plegctl=0;
          _plegstr="";
            
             
             
                 
/*3.level*/ 
                 {LM13_F,LM13_p}=TAR_TEST(_y_,reg2,_auxx);
             
                 if LM13_p >level/4;                /* if NO THIRD BREAK */
                    " pvalue for NO third break "  LM13_p;
                    c_m[1,em+1]=c_m[1,em+1]+1;    /* save em-count */
                    goto replloop;              /* take next replication */
                  else;
                   " pvalue for THIRD BREAK "  LM13_p;
                     em=em+1;
                     c_m[1,em+1]=c_m[1,em+1]+1;
                     
                     {r1}=tar3(_y_,_X_,_tvar_);
                    "New breaks" r1'; 
                     z1=_X_.*(_tvar_.<=r1[1]);
                     z2=_X_.*(_tvar_.>r1[1] .and _tvar_.<=r1[2]);
                     z3=_X_.*(_tvar_.>r1[2] .and _tvar_.<=r1[3]);
                     z4=_X_.*(_tvar_.>r1[3]);
                     
             reg3=z1~z2~z3~z4; 
                           
             b1hat=(invpd(z1'z1)*z1'_y_)';
             b2hat=(invpd(z2'z2)*z2'_y_)';
             b3hat=(invpd(z3'z3)*z3'_y_)';
             b4hat=(invpd(z4'z4)*z4'_y_)';
             bethat=(b1hat~b2hat~b3hat~b4hat)';
             
             u3=(_y_-reg3*bethat)'(_y_-reg3*bethat);
             "SSE3=" u3;
            
          _plctrl={0,0,0};
          _pltype={6,2,5};
          _plegctl=1;
          _plegstr="data\000residuals\000fit TAR3";
          xy(seqa(1,1, rows(u)),_y_~(_y_-reg3*bethat)~(reg3*bethat));
          _plegctl=0;
          _plegstr="";
            
            yyh=trimr(qt,abs(rows(qt) - rows(_y_)),0);       
           y1=miss(_y_.*(_tvar_.<=r1[1]),0);
           y2=miss(_y_.*(_tvar_.>r1[1].and _tvar_.<=r1[2]),0);  
           y3=miss(_y_.*(_tvar_.>r1[2].and _tvar_.<=r1[3]),0);
           y4=miss(_y_.*(_tvar_.>r1[3]),0);
           
           _plctrl={0,-1,-1,-1,-1};
           _pdate=0;
           _pnum=2;
          _pltype={6,1,1,1,1};
          _pstype={1,6,8,4,1};
          _psymsiz={5,4,2,3,4};   
          xy(seqa(1959,1/12,rows(_y_)),_y_~y1~y2~y3~y4);       
           
           yyh1=miss(yyh.*(_tvar_.<=r1[1]),0);
           yyh2=miss(yyh.*(_tvar_.>r1[1].and _tvar_.<=r1[2]),0);
           yyh3=miss(yyh.*(_tvar_.>r1[2].and _tvar_.<=r1[3]),0);
           yyh4=miss(yyh.*(_tvar_.>r1[3]),0);
          _psymsiz={5,4,3,3,4}; 
          _pstype={1,6,8,7,1};
          _plegctl=1;
          _plegstr="Data\000Regime I\000Regime II\000Regime III\000Regime IV";
                 
           xy(seqa(1960,1/12,rows(yyh)),yyh~yyh1~yyh2~yyh3~yyh4);
             
             
             
  /*4.level*/ 
                 {LM14_F,LM14_p}=TAR_TEST(_y_,reg3,_auxx);
                 
                 if LM14_p >level/8;                /* if NO THIRD BREAK */
                    " pvalue for NO Fourth BREAK "  LM14_p;
                    c_m[1,em+1]=c_m[1,em+1]+1;    /* save em-count */
                    
                                     
                    
                    goto replloop;              /* take next replication */
                  else;
                   " pvalue for Fourth BREAK "  LM14_p;
                     em=em+1;
                     c_m[1,em+1]=c_m[1,em+1]+1;
                 
                 
                   endif; /* endif for 4th level lin rejected */                
     
                endif; /* endif for 3.level lin rejected */   
        
           endif;  /* endif for 2.level linearity rejected */
             
       endif; /* endif for 1.level linearity rejected*/
    
     replloop:
     
   output on;
   c_m[1,1:4]~breaks[1,.];
   output off;
 

endfor; /* end levels*/

ttime=time_com(algus);
"Time wasted on this simulation=" ttime;
end;  
                    
           
/**************************************************************************/
/***************************** PROCEDURES *********************************/
/**************************************************************************/ 
/* ======================= PROC LIN_TEST ==================== */
/* INPUT - y,X_0,intercepts_0, auxiliary regressors           */
/* OUTPUT - F-statistics nd its p-value                       */
/* ========================================================== */
proc(2) = LIN_TEST(data,reg,auxx);

local j,y,X,all,nr_restr,nr_regr,df,
LM_pvF,LM_stF,invxm,invnl,res_0,res_1,SSR_0,SSR_1,aux,intc;

y=data;
X=reg;
aux=auxx;

for j (1,cols(X),1);    /* Standardise regressors.*/
   if NOT meanc(X[.,j])==1;   
      X[.,j]=(X[.,j]-meanc(X[.,j]))/stdc(X[.,j]);
   endif;   
endfor;

for j (1,cols(aux),1);    /* standardise auxiliary regressors */
   aux[.,j]=(aux[.,j]-meanc(aux[.,j]))/stdc(aux[.,j]);
endfor;

all=X~aux;
nr_restr=cols(aux);
nr_regr=cols(X);
df=rows(y)-nr_restr-nr_regr;

LM_pvF=miss(0,0);
LM_stF=miss(0,0);

invxm=invpd(X'X);

if NOT scalmiss(invxm);
  res_0=y-X*invxm*X'y;
  SSR_0=res_0'res_0;    

  invnl=invpd(all'all);
  
  if NOT scalmiss(invnl);
     res_1=y-all*invnl*all'y;
     SSR_1=res_1'res_1;
  
     LM_stF=((SSR_0-SSR_1)/nr_restr)/(SSR_1/df);
     LM_pvF=cdffc(LM_stF,nr_restr,df);
  endif;
endif;

retp(LM_stF,LM_pvF);
endp;  

/* ======================= PROC TAR_TEST ================== */
/* INPUT - y,X_0,intercepts_0, auxiliary regressors           */
/* OUTPUT - F-statistics nd its p-value                       */
/* ========================================================== */
proc(2) = TAR_TEST(data,reg,auxx);

local j,y,X,all,nr_restr,nr_regr,df,
LM_pvF,LM_stF,invxm,invnl,res_0,res_1,SSR_0,SSR_1,aux,intc;

y=data;
X=reg;

aux=auxx;

for j (1,cols(X),1);    /* Standardise regressors.*/
   if NOT meanc(X[.,j])==1;   
      X[.,j]=X[.,j]/stdc(X[.,j]);
   endif;   
endfor;


for j (1,cols(aux),1);    /* standardise auxiliary regressors */
   aux[.,j]=aux[.,j]/stdc(aux[.,j]);
endfor;

all=X~aux; 
nr_restr=cols(aux); 
nr_regr=cols(X);    
df=rows(y)-nr_restr-nr_regr;  

LM_pvF=miss(0,0);
LM_stF=miss(0,0);

invxm=invpd(X'X);  

if NOT scalmiss(invxm);
  res_0=y-X*invxm*X'y;
  SSR_0=res_0'res_0; 

  invnl=invpd(all'all);
  
  if NOT scalmiss(invnl); 
     res_1=y-all*invnl*all'y;
     SSR_1=res_1'res_1;
  
     LM_stF=((SSR_0-SSR_1)/nr_restr)/(SSR_1/df);
     LM_pvF=cdffc(LM_stF,nr_restr,df);
  endif;
endif;

retp(LM_stF,LM_pvF);
endp;   

/* ================= PROC TAR ============================= */
/* INPUT - data,regressors,thrs.var,sorted/trimmed thrs.var */
/* OUTPUT - ghat,bhat1,bhat2                                */
/* ======================================================== */
proc (4) = tar(y,x,q,qq);

local x1,x2,gn,s,ghat,j,x1hat,x2hat,bethat,reg,bhat1,bhat2,u;
gn=rows(qq);
s=zeros(gn,1);
    j=1;
    do while j<=gn;
      x1=x.*(q.<=qq[j]);
      x2=x.*(q.>qq[j]);
    s[j]=y'y-y'x1*invpd(x1'x1)*x1'y-y'x2*invpd(x2'x2)*x2'y;
    j=j+1;
    endo;
    
     ghat=qq[minindc(s)];
     x1hat=x.*(q.<=ghat);
     x2hat=x.*(q.>ghat);
     reg=x1hat~x2hat;
     bethat=invpd(reg'reg)*reg'y;
     bhat1=bethat[1:rows(bethat)/2];
     bhat2=bethat[rows(bethat)/2+1:rows(bethat)];
     u=y-reg*bethat;
retp(ghat,bhat1',bhat2',u);
endp;




proc qsort(q);
local j,n,n1,qq,qj,nj,qq1,qq2;
n=rows(q);
n1=round(_trim*n); 
qq1=sortc(q,1);
if rows qq1 le (2*n1);
  qq2=miss(0,0);
 else;
   qq2=qq1[n1+1:rows(qq1)-n1];
endif;

retp(qq2);
endp;

/* ================= PROC TIME_COM =================== */
/* INPUT - time                                        */
/* OUTPUT - total time wasted                          */
/* =================================================== */
proc(1)=time_com(algus); 
/* need to place variable "algus" at the beginning of program */
/* returns time difference between algus and lopp  */
local lopp, vahe;

	lopp = time;
	vahe = lopp-algus;
	
	if vahe[4]<0;
		vahe[4] = vahe[4]+100;
		vahe[3]=vahe[3]-1;
	endif;  
	
	if vahe[3]<0;
		vahe[3] = vahe[3]+60;
		vahe[2]=vahe[2]-1;
	endif;
	
	if vahe[2]<0;
	    vahe[2] = vahe[2]+60;
		vahe[1]=vahe[1]-1;
	endif;  
	
	if vahe[1]<0;
	   vahe[1] = vahe[1]+24;
	endif;  
retp (timestr(vahe));
endp;



proc (4) = tarstep2(y,x,q,rhat1);
local qq1u,qq2u,rhat2,gg,qqcon,qq1,qq2,gg1,gg2,s1,s2,x1,x2,x3,gn1,gn2,s,j,k,
z1,y1,mz1,e1,sig1,z2,y2,mz2,e2,sig2,ic1,ic2,qt1,qt2,mhat21,mhat22,mhat2,
sig,z1hat,z2hat,q1hat,q2hat,m1,m2,g1,g2;

z1hat=x.*(q.<=rhat1);
z2hat=x.*(q.>=rhat1);

q1hat=eye(rows(z1hat))-z1hat*invpd(z1hat'z1hat)*z1hat';
q2hat=eye(rows(z2hat))-z2hat*invpd(z2hat'z2hat)*z2hat';

qq1u=packr(miss(q.*(q.>rhat1),0));
qq2u=packr(miss(q.*(q.<rhat1),0));

qq1=qsort(qq1u);
qq2=qsort(qq2u);

if NOT scalmiss(qq2);
  {m2,g2}=tarr(q2hat'y,q2hat'x,q,qq2);
 else;
  /*"too few obs to form new regime on LEFT-side";*/
 m2=0;
 g2=miss(0,0);
endif;

if NOT scalmiss(qq1);
  {m1,g1}=tarr(q1hat'y,q1hat'x,q,qq1);
 else;
 /* "too few obs to form new regime on RIGHT-side";*/
 m1=0;
 g1=miss(0,0);
endif;

retp(m2,m1,g2,g1); /* report left side first */
endp;

/*****************************************************/

@ TAR Procedures @

/* FITS A TWO REGIME MODEL */

proc (1) = tarr(y,x,q,qq);
local mi,e,gn,s,j,xd,xde,xxd,gg,mmi,ghat;

mi=ginv(moment(x,0));
e=y-x*mi*(x'y);
gn=rows(qq);
s=zeros(gn,1);

   j=1;do while j<=gn;
      gg=qq[j];
      if scalmiss(gg); continue; endif;
      xd=x.*(q.<=gg);
      xxd=moment(xd,0);
      mmi=xxd-xxd*mi*xxd;
      xde=xd'e;
      s[j]=xde'dinv(xde,mmi);
    j=j+1;endo;

  ghat=qq[maxindc(s)];
retp(ghat);
endp;


proc tar_d(y,x,x1,q,qq);
local mi,e,qn,gn,s,j,xd,xde,xxd,gg,mmi,ghat,xdx;
  mi=ginv(moment(x,0));
  e=y-x*mi*(x'y);
  gn=rows(qq);
  s=zeros(gn,1);
    j=1;
    do while j<=gn;
      gg=qq[j];
      if scalmiss(gg); continue; endif;
      xd=x1~(x.*(q.<=gg));
      xxd=moment(xd,0);
      xdx=x'xd;
      mmi=xxd-xdx'mi*xdx;
      xde=xd'e;
      s[j]=xde'dinv(xde,mmi);
    j=j+1;endo;
  ghat=qq[maxindc(s)];
retp(ghat);
endp;



proc (4) = tar2(dat,reg,tv);
local p,y,x,j,n,q,gege2,dhat,ghat,ghat1,ghat2n,ghat2,xx,e,qd,x1,f,gg,etar,
blah1,blah2,u;
n=rows(dat);
p=1;
 /*"******inside TAR2*******";*/
  /*y=dat[p+1:rows(dat)];*/
  y=dat;
  x=reg;
  q = tv;
  {ghat,blah1,blah2,u}=tar(y,x,q,qsort(q));   @ fit TAR-1 model @
  /*"******inside TAR2: tar1 fitted *******";*/
  qd=q;
  x1=x.*(qd.<=ghat);
  ghat2=tar_d(y,x,x1,qd,qsort_2(qd,ghat));  @ fit TAR-2 model @
  gege2=ghat2;
  x1=x.*(qd.<=ghat2);
  ghat1=tar_d(y,x,x1,qd,qsort_2(qd,ghat2));  @ iterated estimate of gamma_1 @
  x1=x.*(qd.<=ghat1);
  ghat2=tar_d(y,x,x1,qd,qsort_2(qd,ghat1));  @ iterated estimate of gamma_2 @

retp(ghat,gege2,ghat1,ghat2);
endp;



proc (1) = tar3(dat,reg,tv);
local p,y,x,j,n,q,gege2,dhat,ghat,ghat1,ghat2n,ghat2,xx,e,qd,x1,f,gg,etar,
blah1,blah2,u,ght,ghat3,ghat31, ghat32,ghat33,g2hat,geehat;
n=rows(dat);
p=1;
 /*"******inside TAR3*******";*/
  /*y=dat[p+1:rows(dat)];*/
  y=dat;
  x=reg;
  q = tv;
  
  {ghat,blah1,blah2,u}=tar(y,x,q,qsort(q));   @ fit TAR-1 model @
  /*"******inside TAR2: tar1 fitted *******";*/
  
  qd=q;
  x1=x.*(qd.<=ghat);
  ghat2=tar_d(y,x,x1,qd,qsort_2(qd,ghat));  @ fit TAR-2 model @
  gege2=ghat2;
  x1=x.*(qd.<=ghat2);
  ghat1=tar_d(y,x,x1,qd,qsort_2(qd,ghat2));  @ iterated estimate of gamma_1 @
  x1=x.*(qd.<=ghat1);
  ghat2=tar_d(y,x,x1,qd,qsort_2(qd,ghat1));  @ iterated estimate of gamma_2 @

  ght=sortc(ghat1|ghat2, 1);
  ghat1=ght[1];
  ghat2=ght[2];
  
   /* Ghat3 iteration */
  
       ghat3=tar_d3(y,x,ght,qd,qsort_3(qd,ght));  @ fit TAR-3 model @
       "ghat3" ghat3;
       
       
       x1=x.*(qd.<=ghat3);
       g2hat=sortc(ghat3|ghat1,1);
       ghat31=tar_d3(y,x,g2hat,qd,qsort_3(qd,g2hat)); @ iterate gamma1 @                       
       "ghat31" ghat31;
       
       g2hat=sortc(ghat31|ghat3,1);
       ghat32=tar_d3(y,x,g2hat,qd,qsort_3(qd,g2hat));  @ iterate gamma_2 @
       "ghat32" ghat32;
       
       g2hat=sortc(ghat31|ghat32,1);
       ghat33=tar_d3(y,x,g2hat,qd,qsort_3(qd,g2hat)); @ iterate gamma3 @ 
       "ghat33" ghat33;
       
       geehat=sortc(ghat31|ghat32|ghat33,1);
       ghat1=geehat[1]; 
       ghat2=geehat[2]; 
       ghat3=geehat[3];
            
       /*"Iterated g1 g2 g3 " ghat1~ghat2~ghat3; */
  

retp(geehat);
endp;


proc tar_d3(y,x,ghat,q,qq);
local mi,e,qn,gn,s,j,xd,xde,xxd,gg,mmi,ghat3,xdx,x1,x2,x3,x4;
  
  mi=ginv(moment(x,0));
  e=y-x*mi*(x'y);
   
  gn=rows(qq);
  s=zeros(gn,1);
  j=1;
  do while j<=gn;
    gg=qq[j];
    if scalmiss(gg); continue; endif;
    ghat3=sortc(ghat|gg,1); /*"ghat3=" ghat3;*/
      
    x1=x.*(q.<=ghat3[1]);
    x2=x.*(q.>ghat3[1] .and q.<=ghat3[2]);
    x3=x.*(q.>ghat3[2] .and q.<=ghat3[3]); 
    x4=x.*(q.>ghat3[3]);
      
    xd=x1~x2~x3~x4; 
    xxd=moment(xd,0);
    xdx=x'xd;
    mmi=xxd-xdx'mi*xdx;
    xde=xd'e;
    s[j]=xde'dinv(xde,mmi);
    j=j+1;
  endo;
   ghat3=qq[maxindc(s)];
 
retp(ghat3);
endp;


/* SOME PROCS */
/*****************************************************/

proc quant(x,q);
local s,qq;
  s=sortc(x,1);
  qq=s[round(rows(s)*q)];
retp(qq);
endp;

proc ginv(m);
local mi;
  trap 1;
  mi=invpd(m);
  trap 0;
  if scalerr(mi);
    mi=pinv(m);
  endif;
retp(mi);
endp;

proc dinv(y,x);
local d;
  trap 1;
    d=y/x;
  trap 0;
  if scalerr(d);
    d=ginv(moment(x,0))*(x'y);
  endif;
retp(d);
endp;

proc qsort_2(q,ghat);
local n,n1,qq,nq;
  n=rows(_y_);
  n1=round(_trim*n);
qq=unique(q,1);
  qq=qq[n1:(rows(qq)-n1+1)];
  qq=delif(qq,(abs(seqa(1,1,rows(qq))-maxindc(qq.>=ghat)).<n1));
  if _qnum > 0;
      nq=rows(qq);
      if nq>_qnum;
        qq=qq[ceil(seqa(1,1,_qnum)./_qnum*nq)];
      endif;
  endif;
retp(qq);
endp;

proc qsort_3(q,ghat);
local qq,qq1,q1,q2,q3,r1,r2,q11,q22;
  
  r1=ghat[1];
  r2=ghat[2];
  qq=unique(q,1);
  
  q1=qqsort(packr(miss(qq.*(qq.<r1),0))); /* if EXACT 0 in the dataset => gets eliminated */
  q2=qqsort(packr(miss(qq.*(qq.>r1 .and qq.<r2),0))); 
  q3=qqsort(packr(miss(qq.*(qq.>r2),0)));
    
  qq1=packr(q1|q2|q3);    
  retp(qq1);
endp;

proc qqsort(q);
 local qqq,aarea,area,n1;
  aarea=sortc(q,1); 
  n=rows(_y_);
  n1=round(_trim*n);
  
                     /* FIX thresh.var. */
  if NOT rows(aarea)<2*n1+1;
     area=trimr(aarea,n1,n1);
     qqq=unique(area,1);
   else;
    qqq=miss(0,0);
    
  endif ;
 retp(qqq);
endp;
